"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const AppCommandRegister_1 = __importDefault(require("./command/AppCommandRegister"));
const GameCommand_1 = __importDefault(require("./command/GameCommand"));
const GameStateManager_1 = __importDefault(require("./state/GameStateManager"));
class TicTacToeBot {
    constructor(configuration, eventHandler) {
        this._configuration = configuration;
        this._eventHandler = eventHandler;
        this.command = new GameCommand_1.default(new GameStateManager_1.default(this));
    }
    get configuration() {
        return this._configuration;
    }
    get eventHandler() {
        return this._eventHandler;
    }
    attachToClient(client) {
        if (this.configuration.slashCommand) {
            const register = new AppCommandRegister_1.default(client, this.configuration.slashCommand);
            client.on('message', register.handleDeployMessage.bind(register));
            client.ws.on('INTERACTION_CREATE', interaction => this.command.handleInteraction(client, interaction));
        }
        if (this.configuration.command) {
            client.on('message', this.command.handleMessage.bind(this.command));
        }
    }
    handleMessage(message) {
        this.command.handleMessage(message, true);
    }
    handleInteraction(interaction, client) {
        this.command.handleInteraction(client, interaction, true);
    }
}
exports.default = TicTacToeBot;
